/*************************************************************
//
// CASSIOPEIA application launcher disable mode Test sample program
// Copyright (C) 1999 CASIO Computer Co.,Ltd.
// All right reserved.
//
// Module:      Disable/Enable Application Launcher Function
// Source       AppModeDraw.cpp
// Abstract: The sample application demonstrates the use of the
// function of application launcher buttons to disable/enable.
//
// This source code is only intended as a supplement to the
// CASIO's application launcher disable mode Reference and related
// the documentation provided with the library.
// See these sources for detailed information regarding the
// Cassiopeia's application launcher disable mode.
//
************************************************************/

#include <windows.h>
#include <commctrl.h>
#include "resource.h"

#include "launcher.h"  // CASSIOPEIA Library: include file

#define APPNAME     (TEXT("AppModeDraw"))
#define WINCLAS     (TEXT("AppModeDraw"))

#define A_BTN   'a'
#define B_BTN   'b'
#define S_BTN   's'
#define V_BTN   'v'

static BOOL fD_LEFT;
static BOOL fD_RIGHT;
static BOOL fD_UP;
static BOOL fD_DOWN;
static BOOL fD_A;
static BOOL fD_B;
static BOOL fD_S;
// static BOOL fD_V;

static unsigned short tmp[64];

#define TEXT_HEIGHT     20
#define PEN_RELEASED     0
#define PEN_PUSHED       1

LRESULT CALLBACK WindowFunc( HWND, UINT, WPARAM, LPARAM );
void clearBoard( HWND hWnd );
void Line( HDC hDc, int sx, int sy, int ex, int ey );

WCHAR   szWinName[]     = WINCLAS;
WCHAR   str[80]         = TEXT("");
static HINSTANCE hInst  = NULL;
UINT    idTimer;

UINT    penStatus = PEN_RELEASED;
int     exX, exY, dX, dY;
int     lcdX, lcdY;
int     BrushThickness;
UINT    CurrCol, PrevCol;

int WINAPI WinMain(
    HINSTANCE hThisInst,
    HINSTANCE hPrevInst,
    LPWSTR lpszArgs,
    int nWinMode
){
    HWND    hwnd;
    MSG msg;
    WNDCLASS wcl;

    // define a window class
    wcl.hInstance     = hThisInst;
    wcl.lpszClassName = szWinName;
    wcl.lpfnWndProc   = WindowFunc;
    wcl.style         = CS_DBLCLKS;
    wcl.hIcon   = NULL;
    wcl.hCursor = NULL;
    wcl.lpszMenuName = NULL;
    wcl.cbClsExtra = 0;
    wcl.cbWndExtra = 0;
    wcl.hbrBackground = (HBRUSH)GetStockObject( BLACK_BRUSH );

    if( RegisterClass(&wcl) == 0 ) return 0;

    hwnd = CreateWindow(
        szWinName,                  // name of window class
        APPNAME,                    // title
        WS_BORDER  | WS_VISIBLE,    // window style -normal
        CW_USEDEFAULT,              // X coordinate -let Windows decide
        CW_USEDEFAULT,              // Y coordinate -let Windows decice
        CW_USEDEFAULT,              // width -let Windows decide
        CW_USEDEFAULT,              // height -let Windows decide
        HWND_DESKTOP,               // no parent window
        NULL,                       // no menu
        hThisInst,                  // handle of this instance of the program
        NULL                        // no additional arguments
    );

    hInst = hThisInst;

    KeyAPI_WinCapsEntry(hwnd, A_BTN, B_BTN, S_BTN, V_BTN);

    ShowWindow( hwnd, nWinMode );
    InvalidateRect(hwnd, NULL, TRUE);
    UpdateWindow( hwnd );

    HWND    hWndCb;
    hWndCb = CommandBar_Create(hInst, hwnd, 1);
    CommandBar_AddAdornments(hWndCb, 0, 0);
    
    clearBoard(hwnd);

    while( GetMessage( &msg, NULL, 0, 0 ) != 0 ) {
        TranslateMessage( &msg );   // allow use of keyboard
        DispatchMessage( &msg );    // return control to Windows
    }

    return msg.wParam;
}

LRESULT CALLBACK WindowFunc(
    HWND hWnd,
    UINT message,
    WPARAM  wParam,
    LPARAM lParam
){
    HDC         hDc;
    PAINTSTRUCT ps;
    RECT        rect;
    HRGN        hRgn;
    HPEN        hPen, hOldPen;
    int         colR, colG, colB;

    switch( message ) {
        case WM_PAINT:
            hDc = BeginPaint(hWnd, &ps);
            EndPaint(hWnd, &ps);
            clearBoard( hWnd );
            break;
        case WM_KEYDOWN:
            switch (wParam){
                case VK_RETURN:
                    if (BrushThickness < 5) BrushThickness++;
                    colR = colG = colB = 156;
                    if (fD_S) colR = 255;
                    if (fD_A) colB = 255;
                    if (fD_B) colG = 255;
                    CurrCol = (colR << 16) + (colG << 8) + (colB);

                    wsprintf(tmp, TEXT(" Thickness : %2d    (x, y) = (%3d, %3d)   "),
                             BrushThickness, (exX + dX), (exY + dY));
                    hDc = GetDC( hWnd );

                    GetClipBox(hDc, &rect);
                    SetBkColor(hDc, RGB(colR, colG, colB));
                    DrawText(hDc, TEXT("          "), -1, &rect, DT_RIGHT);
                    DrawText(hDc, tmp, -1, &rect, DT_LEFT);

                    PrevCol = CurrCol;
                    ReleaseDC(hWnd, hDc);
                    break;
                case VK_ESCAPE:
                    if (BrushThickness > 2) BrushThickness--;
                    colR = colG = colB = 156;
                    if (fD_S) colR = 255;
                    if (fD_A) colB = 255;
                    if (fD_B) colG = 255;
                    CurrCol = (colR << 16) + (colG << 8) + (colB);

                    wsprintf(tmp, TEXT(" Thickness : %2d    (x, y) = (%3d, %3d)   "),
                             BrushThickness, (exX + dX), (exY + dY));
                    hDc = GetDC( hWnd );

                    GetClipBox(hDc, &rect);
                    SetBkColor(hDc, RGB(colR, colG, colB));
                    DrawText(hDc, TEXT("          "), -1, &rect, DT_RIGHT);
                    DrawText(hDc, tmp, -1, &rect, DT_LEFT);

                    PrevCol = CurrCol;
                    ReleaseDC(hWnd, hDc);
                    break;
                case VK_LEFT:   fD_LEFT  = TRUE;    break;
                case VK_RIGHT:  fD_RIGHT = TRUE;    break;
                case VK_DOWN:   fD_DOWN  = TRUE;    break;
                case VK_UP:     fD_UP    = TRUE;    break;
                case A_BTN:     fD_A     = TRUE;    break;
                case B_BTN:     fD_B     = TRUE;    break;
                case S_BTN:     fD_S     = TRUE;    break;
                case V_BTN:
                    // fD_V = TRUE;
                    clearBoard( hWnd );
                    break;
            }
            break;
        case WM_KEYUP:
            switch (wParam){
                case VK_LEFT:   fD_LEFT  = FALSE;   break;
                case VK_RIGHT:  fD_RIGHT = FALSE;   break;
                case VK_DOWN:   fD_DOWN  = FALSE;   break;
                case VK_UP:     fD_UP    = FALSE;   break;
                case A_BTN:     fD_A     = FALSE;   break;
                case B_BTN:     fD_B     = FALSE;   break;
                case S_BTN:     fD_S     = FALSE;   break;
                case V_BTN:
                    // fD_V = FALSE;
                    break;
            }
            break;
        case WM_CHAR:
            break;
        case WM_CREATE:
            lcdX = GetSystemMetrics( SM_CXSCREEN );
            lcdY = GetSystemMetrics( SM_CYSCREEN ) - 25;
            break;
        case WM_LBUTTONDOWN:
            penStatus = PEN_PUSHED;
            exX = LOWORD(lParam);
            exY = HIWORD(lParam);

            colR = colG = colB = 156;
            if (fD_S) colR = 255;
            if (fD_A) colB = 255;
            if (fD_B) colG = 255;
            CurrCol = (colR << 16) + (colG << 8) + (colB);

            hDc = GetDC( hWnd );
            GetClipBox(hDc, &rect);

            if (exX <= 0)        exX = 0;
            else if (exX >= (lcdX - 4)) exX = lcdX - 4;
            if (exY <= (rect.top + ((TEXT_HEIGHT + 1) * 2))) exY = rect.top + ((TEXT_HEIGHT + 1) * 2);
            else if (exY >= (lcdY - 4)) exY = lcdY - 4;
            wsprintf(tmp, TEXT(" Thickness : %2d    (x, y) = (%3d, %3d)   "),
                     BrushThickness, exX, exY);

            SetBkColor(hDc, RGB(colR, colG, colB));
            DrawText(hDc, TEXT("          "), -1, &rect, DT_RIGHT);
            DrawText(hDc, tmp, -1, &rect, DT_LEFT);

            PrevCol = CurrCol;
            ReleaseDC(hWnd, hDc);

            if (!idTimer) idTimer = SetTimer(hWnd, 1, 25, NULL);
            break;          
        case WM_LBUTTONUP:
            penStatus = PEN_RELEASED;
        case WM_MOUSEMOVE:
        case WM_LBUTTONDBLCLK:
            exX = LOWORD(lParam);
            exY = HIWORD(lParam);

            colR = colG = colB = 156;
            if (fD_S) colR = 255;
            if (fD_A) colB = 255;
            if (fD_B) colG = 255;
            CurrCol = (colR << 16) + (colG << 8) + (colB);

            hDc = GetDC( hWnd );
            GetClipBox(hDc, &rect);

            if (exX <= 0) exX = 0;
            else if (exX >= (lcdX - 4)) exX = lcdX - 4;
            if (exY <= (rect.top + ((TEXT_HEIGHT + 1) * 2))) exY = rect.top + ((TEXT_HEIGHT + 1) * 2);
            else if (exY >= (lcdY - 4)) exY = lcdY - 4;

            SetBkColor(hDc, RGB(colR, colG, colB));
            DrawText(hDc, TEXT("          "), -1, &rect, DT_RIGHT);
            DrawText(hDc, tmp, -1, &rect, DT_LEFT);

            PrevCol = CurrCol;
            ReleaseDC(hWnd, hDc);
            break;
        case WM_TIMER:
            colR = colG = colB = 156;
            if (fD_LEFT)    dX -= 2;
            if (fD_RIGHT)   dX += 2;
            if (fD_DOWN)    dY += 2;
            if (fD_UP)      dY -= 2;
            if (fD_S)       colR = 255;
            if (fD_A)       colB = 255;
            if (fD_B)       colG = 255;
            CurrCol = (colR << 16) + (colG << 8) + (colB);

            wsprintf(tmp, TEXT(" Thickness : %2d    (x, y) = (%3d, %3d)   "),
                     BrushThickness, (exX + dX), (exY + dY));

            hDc = GetDC( hWnd );

            hPen    = CreatePen(PS_SOLID, BrushThickness, RGB(colR, colG, colB));
            hOldPen = (HPEN) SelectObject(hDc, hPen);

            GetClipBox(hDc, &rect);
            if ((CurrCol != PrevCol) || ((dX != 0) || (dY != 0)))  {
                SetBkColor(hDc, RGB(colR, colG, colB));
                DrawText(hDc, TEXT("          "), -1, &rect, DT_RIGHT);
                DrawText(hDc, tmp, -1, &rect, DT_LEFT);
            }

            rect.top += (TEXT_HEIGHT + 1) * 2;

            if ((exX + dX) <= 0) dX = 0;
            else if ((exX + dX) >= (lcdX - 4)) { exX = lcdX - 4; dX = 0; }
            if ((exY + dY) <= rect.top) dY = 0;
            else if ((exY + dY) >= (lcdY - 4)) { exY = lcdY - 4; dY = 0; }
            
            if ((dX != 0) || (dY != 0)) {
                hRgn = CreateRectRgnIndirect( &rect );
                SelectClipRgn(hDc, hRgn);

                Line( hDc, exX, exY, exX + dX, exY + dY);
            }
            
            SelectObject(hDc, hOldPen);
            DeleteObject(hPen);

            exX = exX + dX;
            exY = exY + dY;
            dX = dY = 0;
            PrevCol = CurrCol;

            ReleaseDC(hWnd, hDc);
            break;
        case WM_DESTROY:

            // CASSIOPEIA Library:
            // Exit launcher disable mode and enable keys to launch applications.
            KeyAPI_WinCapsDelete(hWnd);

            PostQuitMessage( 0 );
            break;
        case WM_ACTIVATE:

            // CASSIOPEIA Library:
            // Enter launcher disable mode.
            KeyAPI_WinCapsEntry(hWnd, A_BTN, B_BTN, S_BTN, V_BTN);

        default:
            return DefWindowProc( hWnd, message, wParam, lParam );
        }
    return 0;
}

void clearBoard(HWND hWnd) {
    HDC     hDc, hMemDc;
    RECT    rect;
    HBITMAP hBitmap;

    KillTimer(hWnd, idTimer);
    idTimer = 0;

    hDc     = GetDC( hWnd );
    hBitmap = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP_VOICE));
    hMemDc  = CreateCompatibleDC(hDc);
    SelectObject(hMemDc, hBitmap);

    GetClipBox(hDc, &rect);
    PatBlt(hDc, rect.left, rect.top, lcdX, rect.top + TEXT_HEIGHT, WHITENESS);
    DrawText(hDc, TEXT(" Tap the window, and push buttons"), -1, &rect, DT_LEFT);
    rect.top += TEXT_HEIGHT + 1;
    DrawText(hDc, TEXT("      Clear Disp.      Red      Blue       Green"), -1, &rect, DT_LEFT);
    BitBlt(hDc, rect.left + 5, rect.top, 16, 16, hMemDc, 0, 0, SRCCOPY);
    hBitmap = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP_LAUNCHER));
    SelectObject(hMemDc, hBitmap);
    BitBlt(hDc, rect.left + 85, rect.top, 16, 16, hMemDc, 0, 0, SRCCOPY);
    hBitmap = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP_CALENDAR));
    SelectObject(hMemDc, hBitmap);
    BitBlt(hDc, rect.left + 130, rect.top, 16, 16, hMemDc, 0, 0, SRCCOPY);
    hBitmap = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP_CONTACTS));
    SelectObject(hMemDc, hBitmap);
    BitBlt(hDc, rect.left + 180, rect.top, 16, 16, hMemDc, 0, 0, SRCCOPY);
    rect.top += TEXT_HEIGHT + 1;
    PatBlt(hDc, rect.left, rect.top, lcdX, lcdY-TEXT_HEIGHT, BLACKNESS);
    PrevCol = 0xffffff;
    exX = 120; exY = 160;
    BrushThickness = 2;

    ReleaseDC(hWnd, hDc);
    DeleteDC(hMemDc);
    DeleteObject(hBitmap);
}

void Line( HDC hDc, int sx, int sy, int ex, int ey ) {
    POINT linePt[2];

    linePt[0].x = sx;
    linePt[0].y = sy;
    linePt[1].x = ex;
    linePt[1].y = ey;

    Polyline( hDc, linePt, 2 );
}
